/*
	Copyright (C) 2011 Micha Hersch, University of Lausanne, Swiss Institute of Bioinformatics

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *	\file imgGrabber.cpp
 *	\author Basilio Noris, modified by Micha Hersch 
 *	\date
 */

#include <stdio.h>
#include "ip_public.h"
#include "imgGrabber.h"

#ifndef LINUX
#pragma warning(disable: 4996)
#endif

ImgGrabber::ImgGrabber(const char *firstname, f32 frate)
{
	if(!firstname) return;
	u32 len = strlen(firstname);
	if (len < 5) return;
	fnExtension = new char[5];
	filename = new char[len+1];
	sprintf(fnExtension, "%s",&firstname[len-4]);
	strcpy(filename, firstname);

	fnNumFormat = 0;
	FOR(i,len-4){
		char c = filename[len-5 - i];
		if(c < '0' || c > '9') break;
		fnNumFormat++;
	}
	//	printf("%d\n",fnNumFormat);
	char minnum[5];
	memcpy(minnum,firstname+len-4-fnNumFormat,fnNumFormat);
	  minnum[fnNumFormat]='\0';
	  currentFrame = atoi(minnum);
	  numStart = currentFrame;
	  
	  char spec[30];
	FILE *f;
	f = fopen(filename, "rb");
	if(!f)printf("cannot find file %s\n",filename);
	u32 filecount = numStart;
	length = 0;
	numStart = 0;
	while(f){
		fclose(f);
		filecount++;
		length++;
		if(filecount == 1000){
			char *newFilename = new char[255];
			filename[length-4-fnNumFormat] = '\0';
			len++;
			sprintf(newFilename,"%s1000",filename);
			filename = newFilename;
			filecount = 0;
		}
		filename[len-4-fnNumFormat] = '\0';
		sprintf(spec,"%%s%%.%dd%%s",fnNumFormat);
		//	printf("%s\n",spec);
		sprintf(filename,spec,filename, filecount, fnExtension);
		f = fopen(filename, "r");
	}
	//	printf("%s length %d\n",filename, filecount);
	strcpy(filename, firstname);
	IplImage *im = cvLoadImage(filename);
	width = im->width;
	height = im->height;
	framerate = frate;
	if(!length){fprintf(stderr,"cannot open file %s\n",firstname);}
	if(length==1){currentFrame=0;}
	if(length>1){
	  fnBase = new char[len-4-fnNumFormat+1];
	  memcpy(fnBase, filename, len-4-fnNumFormat);
	  fnBase[len-4-fnNumFormat] = '\0';
	  // not necessary to do twice
	  char minnum[5];
	  memcpy(minnum,firstname+len-4-fnNumFormat,fnNumFormat);
	  minnum[fnNumFormat]='\0';
	  currentFrame = atoi(minnum);
	  numStart = currentFrame;
	//	printf("%s %d\n",minnum,currentFrame);
	  cvReleaseImage(&im); im = NULL;
	  bLoop = false;
	}
}

void ImgGrabber::SetPosition(f32 percentage)
{
	percentage = TRIM(percentage, 0, 1.f);
	currentFrame = numStart+(u32)(length * percentage);
}

void ImgGrabber::Skip(u32 frames)
{
	currentFrame += frames;
	if (currentFrame >= length+numStart) currentFrame = length+numStart-1;
}

void ImgGrabber::SetFrame(u32 frame)
{
  if(frame>=numStart && frame<numStart+length) currentFrame = frame;
}

void ImgGrabber::GrabFrame(IplImage **frame, u32 index)
{
	if(currentFrame >= length+numStart) return;
	if(length>1){
	  char format[255];
	  sprintf(format, "%%s%%.%dd%%s",fnNumFormat);
	  sprintf(filename, format, fnBase, currentFrame, fnExtension);
	  currentFrame++;
	}	 
	IplImage *f = (*frame);
	// hack to avoid undefined pointers or freed pointers 
	if(f){
	  cvReleaseImage(&f); f = NULL;
	}
	f = cvLoadImage(filename);
	printf("%s\n",filename);
	(*frame) = f;
	if(bLoop && currentFrame >= length+numStart) currentFrame = numStart;
}

void ImgGrabber::Kill()
{
  delete filename;
  delete fnExtension;
  if(length>1){
    delete fnBase;
  }
}

f32 ImgGrabber::GetFramerate()
{
	return framerate;
}

CvSize ImgGrabber::GetSize()
{
	return cvSize(width, height);
}

f32 ImgGrabber::GetPercentage()
{
	return f32(currentFrame) / length;
}

u32 ImgGrabber::GetLength()
{
	return length;
}

u32 ImgGrabber::GetCurrentFrame()
{
	return currentFrame;
}

void ImgGrabber::SetLoop(bool loop)
{
	bLoop = loop;
}

bool ImgGrabber::GetLoop()
{
	return bLoop;
}


bool ImgGrabber::IsDone()
{
	return currentFrame >= length+numStart;
}


u32 ImgGrabber::GetCurrentTimeIndex()
{
	return currentFrame-numStart;
}
